/****************************************************************************
*
*   Copyright (c) 2005 Dave Hylands
*
*   This program is free software; you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation; either version 2 of the License, or
*   (at your option) any later version.
*
*   This program is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with this program; if not, write to the Free Software
*   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*
****************************************************************************/
/**
*
*   @file   Hardware.h
*
*   @brief  Defines all of the hardware definitions for the chip.
*
****************************************************************************/

#if !defined( HARDWARE_H )
#define HARDWARE_H            /**< Include Guard                           */

/* ---- Include Files ---------------------------------------------------- */

#include "Config.h"

#include <avr/io.h>

// Port B.4 -- Robostix Onboard LED
#define YELLOW_LED_PIN  4
#define YELLOW_LED_MASK ( 1 << YELLOW_LED_PIN )
#define YELLOW_LED_DDR  DDRB
#define YELLOW_LED_PORT PORTB
//--------------------------------------------------------------------------

/*
// Port C.0 (LED Board power supply)
#define BOARD_LED_PIN   0
#define BOARD_LED_MASK  ( 1 << BOARD_LED_PIN )
#define BOARD_LED_DDR   DDRC
#define BOARD_LED_PORT  PORTC

// Port C.1 (Sensor Board power supply)
#define BOARD2_LED_PIN   1
#define BOARD2_LED_MASK  ( 1 << BOARD2_LED_PIN )
#define BOARD2_LED_DDR   DDRC
#define BOARD2_LED_PORT  PORTC
*/


// Port C.0 (LED Board power supply)
#define LED_BOARD_DEVICE_PIN   0
#define LED_BOARD_DEVICE_MASK  ( 1 << LED_BOARD_DEVICE_PIN )
#define LED_BOARD_DEVICE_DDR   DDRC
#define LED_BOARD_DEVICE_PORT  PORTC

// Port C.1 (Sensor Board power supply)
#define SENSOR_BOARD_DEVICE_PIN   1
#define SENSOR_BOARD_DEVICE_MASK  ( 1 << SENSOR_BOARD_DEVICE_PIN )
#define SENSOR_BOARD_DEVICE_DDR   DDRC
#define SENSOR_BOARD_DEVICE_PORT  PORTC

// Port C.2 (CO sensor heater)
#define CO_HEATER_DEVICE_PIN   2
#define CO_HEATER_DEVICE_MASK  ( 1 << CO_HEATER_DEVICE_PIN )
#define CO_HEATER_DEVICE_DDR   DDRC
#define CO_HEATER_DEVICE_PORT  PORTC

// Port C.3 (CO sensor)
#define CO_SENSOR_DEVICE_PIN   3
#define CO_SENSOR_DEVICE_MASK  ( 1 << CO_SENSOR_DEVICE_PIN )
#define CO_SENSOR_DEVICE_DDR   DDRC
#define CO_SENSOR_DEVICE_PORT  PORTC


// Port C.4
// Port C.5

// Port C.6 (GPS Fix LED - Red)
#define GPS_LED_PIN   6
#define GPS_LED_MASK  ( 1 << GPS_LED_PIN )
#define GPS_LED_DDR   DDRC
#define GPS_LED_PORT  PORTC

// Port C.7 (Sensor preheat period LED - Red)
#define SENSOR_PREHEAT_LED_PIN   7
#define SENSOR_PREHEAT_LED_MASK  ( 1 << SENSOR_PREHEAT_LED_PIN )
#define SENSOR_PREHEAT_LED_DDR   DDRC
#define SENSOR_PREHEAT_LED_PORT  PORTC


// Port F is the ADC port
// Testing Pin 5(i.e. 4)
//#define USB_LED_PIN	4
//#define USB_LED_MASK	( 1 << USB_LED_PIN )
//#define USB_LED_DDR	DDRF
//#define USB_LED_PORT	PORTF


// Port G.0 (System ready LED - Green)
#define SYS_LED_PIN   0
#define SYS_LED_MASK  ( 1 << SYS_LED_PIN )
#define SYS_LED_DDR   DDRG
#define SYS_LED_PORT  PORTG

// Port G.1 (USB Stick in use LED - Red)
#define USB_RED_LED_PIN   1
#define USB_RED_LED_MASK  ( 1 << USB_RED_LED_PIN )
#define USB_RED_LED_DDR   DDRG
#define USB_RED_LED_PORT  PORTG

// Port G.2 (USB Stick can be removed - Green)
#define USB_GREEN_LED_PIN   2
#define USB_GREEN_LED_MASK  ( 1 << USB_GREEN_LED_PIN )
#define USB_GREEN_LED_DDR   DDRG
#define USB_GREEN_LED_PORT  PORTG

// Port G.3 -- Robostix Onboard LED
#define BLUE_LED_PIN    3
#define BLUE_LED_MASK   ( 1 << BLUE_LED_PIN )
#define BLUE_LED_DDR    DDRG
#define BLUE_LED_PORT   PORTG

// Port G.4 -- Robostix Onboard LED
#define RED_LED_PIN     4
#define RED_LED_MASK    ( 1 << RED_LED_PIN )
#define RED_LED_DDR     DDRG
#define RED_LED_PORT    PORTG


//--------------------------------------------------------------------------
// Some convenience macros to turn the LEDs on/off.
//
//  Usage:  LED_ON( BLUE );
//
// to turn on the blue LED.
//
// Note: Setting the pin to 0 turns the LED on.

#define LED_ON( color )		do { color ## _LED_PORT &= ~color ## _LED_MASK; } while (0)
#define LED_OFF( color )	do { color ## _LED_PORT |=  color ## _LED_MASK; } while (0)

#define DEVICE_OFF( dev_name )	do { dev_name ## _DEVICE_PORT &= ~dev_name ## _DEVICE_MASK; } while (0)
#define DEVICE_ON( dev_name )	do { dev_name ## _DEVICE_PORT |=  dev_name ## _DEVICE_MASK; } while (0)


//--------------------------------------------------------------------------
// Port registers.
//
// Note: For input ports, writing a 1 to the PORT register casues an internal
//       pullup resistor to be activated. This feature also requires the 
//       PUD bit to be set in the SFIOR register.
//
// For the DDR pins, 0 = input, 1 = output

// Port A if the one that's just pads on the bottom of the board

#define PORTA_INIT      0xFF
#define DDRA_INIT       0x00

// Port B has 3 PWM lines, and SPI. Pin 4 is connected to the Yellow LED
//
//  7 - OC2     ATM_PWM1C
//  6 - OC1B    ATM_PWM1B
//  5 - OC1A    ATM_PWM1A
//  4 - OC0     PB4 (Yellow LED)
//  3 - MISO    ATM_MISO
//  2 - MOSI    ATM_MOSI
//  1 - SCK     ATM_SCK
//  0 - SS      ATM_SS

#define PORTB_INIT      0xFF    // Enable pullups for inputs, Yellow LED off
#define DDRB_INIT       YELLOW_LED_MASK

// Port C
//
// Port C is available on the headers as 8 I/O lines. We'll assume all inputs
// for now.

#define PORTC_INIT      0xFF    // enable pullups for all inputs
#define DDRC_INIT       0xFF	// Data Direction Register for port C (0000 0000)
//#define DDRC_INIT	(SENSOR_PREHEAT_LED_MASK | GPS_LED_MASK | BOARD_LED_MASK | BOARD2_LED_MASK)
// | LED_BOARD_DEVICE_MASK | SENSOR_BOARD_DEVICE_MASK | CO_HEATER_DEVICE_MASK | CO_SENSOR_DEVICE_MASK) // Selects pins for output

// Port D
//
//  7 - T2      ATM_T2
//  6 - T1      ATM_T1
//  5 - PD5     ATM_PD5
//  4 - IC1     ATM_IC1
//  3 - INT3    ATM_TX1
//  2 - INT2    ATM_RX1
//  1 - INT1    ATM_SDA
//  0 - INT0    ATM_SCL

#define PORTD_INIT      0xFF    // enable pullups for all inputs
#define DDRD_INIT       0x00

// Port E
//
//  7 - INT7    ATM_INT7
//  6 - INT6    ATM_INT6
//  5 - OC3C    ATM_PWM3C
//  4 - OC3B    ATM_PWM3B
//  3 - OC3A    ATM_PWM3A
//  2 - PE2     ATM_IRQ     Used to interrupt gumstix
//  1 - TXD     ATM_TX0
//  0 - RXD     ATM_RX0

#define PORTE_INIT      0xFF    // enable pullups for all inputs
#define DDRE_INIT       0x00

// Port F - A/D lines - could be used as GPIO
//
// We disable all pullups by default so they don't mess with the A/D

#define PORTF_INIT      0x00    // disable pullups for A/D channels
#define DDRF_INIT       0x00	//data direction register for port f
//#define DDRF_INIT	( USB_LED_MASK | 0x00 )

//#define DDRF_INIT	0xFF	// 0x10 4th pin is output(starting from zero)

// Port G
//
//  7 - N/A
//  6 - N/A
//  5 - N/A
//  4 - TOSC1   ATM_PG4 - Red LED
//  3 - TOSC2   ATM_PG3 - Blue LED
//  2 - ALE     ATM_PG2
//  1 - /RD     ATM_PG1
//  0 - /WR     ATM_PG0

#define PORTG_INIT  0xFF    // Enable pullups for inputs, LEDs off
// Makes pins outputs by setting the Data Direction Register for port G pin to 1
#define DDRG_INIT   (RED_LED_MASK | BLUE_LED_MASK | SYS_LED_MASK | USB_RED_LED_MASK | USB_GREEN_LED_MASK)    // Make LEDs outputs

//--------------------------------------------------------------------------
// ASSR - Asynchronous Status Register
//
// TOSC1 & TOSC2 are connected to the Red/Blue LEDs so we need to set
// AS0 to 0. AS0 is the only writable bit.

#define ASSR_INIT   0x00

//-------------------------------------------------------------------------
// ADC Settings

#define ADC_PRESCALAR_2     (( 0 << ADPS2 ) | ( 0 << ADPS1 ) | ( 1 << ADPS0 ))
#define ADC_PRESCALAR_4     (( 0 << ADPS2 ) | ( 1 << ADPS1 ) | ( 0 << ADPS0 ))
#define ADC_PRESCALAR_8     (( 0 << ADPS2 ) | ( 1 << ADPS1 ) | ( 1 << ADPS0 ))
#define ADC_PRESCALAR_16    (( 1 << ADPS2 ) | ( 0 << ADPS1 ) | ( 0 << ADPS0 ))
#define ADC_PRESCALAR_32    (( 1 << ADPS2 ) | ( 0 << ADPS1 ) | ( 1 << ADPS0 ))
#define ADC_PRESCALAR_64    (( 1 << ADPS2 ) | ( 1 << ADPS1 ) | ( 0 << ADPS0 ))
#define ADC_PRESCALAR_128   (( 1 << ADPS2 ) | ( 1 << ADPS1 ) | ( 1 << ADPS0 ))

#define ADCSR_INIT  (( 1 << ADEN ) | ( 1 << ADSC ) | ADC_PRESCALAR_128 )

//--------------------------------------------------------------------------
// UART settings

// TODO: Change baud rate and etc is necessary. I think LED board setting is: ------------------------
// 8 data bits, 1 stop bit, no hardware flow-control and no software handshaking
// baud rate 9600

#define UART0_BAUD_RATE     38400
#define UART1_BAUD_RATE     9600

#define UART_DATA_BIT_8  (( 1 << UCSZ1 ) | ( 1 << UCSZ0 ))
#define UART_PARITY_NONE (( 0 << UPM1 )  | ( 0 << UPM0 ))
#define UART_STOP_BIT_1  ( 1 << USBS )

#define UBRR0_INIT   (( CFG_CPU_CLOCK / 16 / UART0_BAUD_RATE ) - 1 )
#define UBRR1_INIT   (( CFG_CPU_CLOCK / 16 / UART1_BAUD_RATE ) - 1 )

#define UCSR0A_INIT  0
#define UCSR0B_INIT  (( 1 << RXCIE ) | ( 1 << RXEN ) | ( 1 << TXEN ))
#define UCSR0C_INIT  ( UART_DATA_BIT_8 | UART_PARITY_NONE | UART_STOP_BIT_1 )

#define UCSR1A_INIT  0
#define UCSR1B_INIT  (( 1 << RXCIE ) | ( 1 << RXEN ) | ( 1 << TXEN ))
#define UCSR1C_INIT  ( UART_DATA_BIT_8 | UART_PARITY_NONE | UART_STOP_BIT_1 )

/* ---- Variable Externs ------------------------------------------------- */

/**
 *  Description of variable.
 */

/* ---- Function Prototypes ---------------------------------------------- */

extern inline void BeginCritical( void ) { asm volatile("cli\n"); }
extern inline void EndCritical( void ) { asm volatile("sei\n"); }

void InitHardware( void );


/** @} */

#endif // HARDWARE_H

