// TODO: add metadata and license text

#include "logger.h"
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <errno.h>

unsigned logger_level = ~(0U);  // enable all bits

void log_msg(FILE *fp, unsigned level, const char *format, ...)
{
    va_list args;
    
    if(!ISLOGGED(level))        /* do nothing if not logging at this level */
        return;
    
    va_start(args, (char *) format);
    
    if(fp == NULL)              /* default to stderr, if no output stream */
        fp = stderr;
    
    if(format == NULL)
        format = "";
    
    vfprintf(fp, format, args);
    fflush(fp);
    va_end(args);
}

void log_err(FILE *fp, unsigned level, const char *format, ...)
{
    va_list args;

    if(!ISLOGGED(level))        /* do nothing if not logging at this level */
        return;
    
    va_start(args, (char *) format);
    
    if(fp == NULL)              /* default to stderr, if no output stream */
        fp = stderr;
    
    if(format == NULL)
        format = "";
    
    fprintf(fp, "ERROR %d: %s\n", errno, strerror(errno));
    vfprintf(fp, format, args);
    fflush(fp);
    va_end(args);
}

void log_hex(FILE *fp, unsigned level, const void *ptr, unsigned length)
{
    unsigned char *byteptr = (unsigned char *) ptr;
    int i;
    
    if(!ISLOGGED(level))        /* do nothing if not logging at this level */
        return;
    
    if(fp == NULL)              /* default to stderr, if no output stream */
        fp = stderr;
    
    for(i = 0; i < length; i++) {
        if(i % 2 == 0)          /* space out every two bytes */
            fprintf(fp, " ");
        
        fprintf(fp, "%02X", byteptr[i]);    /* print hex byte */
    }
    fprintf(fp, "\n");
    fflush(fp);
}
