// TODO: add metadata and license text

#ifndef LL_H
#define LL_H

#include <stdio.h>

#if defined(__cplusplus)
extern "C" {
#endif
    
extern unsigned logger_level;

/* Default logger levels */
#ifndef LL_FATAL
#define LL_FATAL        0x0001
#endif
#ifndef LL_ERROR
#define LL_ERROR        0x0002
#endif
#ifndef LL_WARN
#define LL_WARN         0x0004
#endif
#ifndef LL_INFO
#define LL_INFO         0x0008
#endif
#ifndef LL_DEBUG
#define LL_DEBUG        0x0010
#endif
#ifndef LL_TRACE
#define LL_TRACE        0x0020
#endif
    
void log_msg(FILE *fp, unsigned level, const char *format, ...);
void log_err(FILE *fp, unsigned level, const char *format, ...);
void log_hex(FILE *fp, unsigned level, const void *ptr, unsigned length);

/* Handy macros (taken from <s.bhatti@cs.ucl.ac.uk>, Nov 2001 */
#define ISLOGGED(lev) \
        ((logger_level & (lev)) != 0)

#define LOG_INFO(lev, str) \
        log_msg(stdout, lev, (char *) (str))

#define LOG_ERROR(lev, str) \
        log_err(stderr, lev, (char *) (str))

#define LOG_TRACE(lev, str) \
        log_err(stderr, lev, "file=%s line=%d %s", \
                __FILE__, __LINE__, (char *) (str))

#define LOG_HEXDUMP(lev, ptr, len) \
        log_hex(stderr, lev, (void *) (ptr), len)

#if defined(__cplusplus)
}
#endif

#endif /* LL_H */
