/****************************************************************************
FERAL ROBOT - URBAN TAPESTRIES (http://urbantapestries.org/)
Linux-based robot client for gumstix platform
Author: Dima Diall
****************************************************************************/

#include "adchw.h"
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/time.h>
#include <sys/select.h>

// ASCII serial protocol parameters to access ADC on robostix
#define ADC_NUM_BASE        16
//TODO: adjust max delay-------------------------------- due to CO sensor ----------
#define MAX_ADC_DELAY       100000
#define MAX_ADC_DATA        20

int read_sensor(int adc_fd, int channel, uint16_t *reading)
{
    char command = channel + '0';
    int result;						// holds result of operations on file descriptor(robostix adc serial port)
    int nfds;						// ???
    fd_set rset;					// file descriptor set
    struct timeval timeout;
    char buffer[MAX_ADC_DATA + 1];
    
#ifdef DUMMY_ADC
// If robostix ADC is not present, this enables testing remaining parts
#warning Building with DUMMY_ADC...
    *reading = 0x03FF;
    return 0;
#else
	// Writes 'character' to serial port connected to robostix(ADC)
    result = write(adc_fd, &command, sizeof(command));
    if(result <= 0) { return -1; }

    FD_ZERO(&rset);
    FD_SET(adc_fd, &rset); //sets testing for adc_fd
    timeout.tv_sec = 10; // seconds
    timeout.tv_usec = MAX_ADC_DELAY; // milli-seconds
    //waits for data to become available from robostix
    result = select(adc_fd + 1, &rset, NULL, NULL, &timeout);
    if(result <= 0) { return -1; } // time out occurs

    // Reads from the serial port connected to the robostix(ADC)
	result = read(adc_fd, buffer, MAX_ADC_DATA);
    if(result <= 0) { return -1; }

    buffer[result] = '\0'; //ends string
    // converts string to unsigned long integer
    *reading = (uint16_t) strtoul(buffer, NULL, ADC_NUM_BASE);
    return 0;
#endif
}
