/****************************************************************************
FERAL ROBOT - URBAN TAPESTRIES (http://urbantapestries.org/)
Linux-based robot client for gumstix platform
Author: Dima Diall
****************************************************************************/

#ifndef SENSORDEF_H
#define SENSORDEF_H

#include "adchw.h"
#include <stdlib.h>

// Function pointer to convert ADC discrete output into sensor's real value.
typedef float (*sensor_adc_conv_fp)(uint16_t adc_val);

// Function pointer to sensor's real value processing handler.
typedef int   (*sensor_val_proc_fp)(float sensor_val);

// Sensor definition entry, which corresponds to an ADC channel.
//     is_active       whether ADC input is connected to a sensor
//     sensor_id       string describing database sensor id (4 chars)
//     adc_handler     pointer to ADC output conversion function
//     value_handler   pointer to sensor value processing handlers
//     transmit        whether to transmit sensor value to UT server
typedef struct {
    short enabled;
    char *sensor_id;
    sensor_adc_conv_fp adc_handler;
    sensor_val_proc_fp value_handler;
    short transmit;
} sensor_t;

// Sensor definition table, one entry for each ADC channel or sensor.
extern const sensor_t sensor_defs[ADC_CHANNELS];

#endif /* SENSORDEF_H */
